#ifndef __DEBUG_CONSOLE_HEADER_FILE__
#define __DEBUG_CONSOLE_HEADER_FILE__

// Includes
#include <Application.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>

// Message Types
#define DC_START 1
#define DC_MESSAGE 2
#define DC_ERROR 3
#define DC_WARNING 4
#define DC_END 5
#define DC_SUCCESS 6

#define DEBUGCONSOLE_ADDMESSAGE 50

// Shortcut functions
#define TRACE __DC.SetColor (0, 0, 0);__DC.DebugMessage
#define TRACE_MSG __DC.SetColor (0 ,0 ,0);__DC.DebugMessage
#define TRACE_ERR __DC.SetColor (255,0,0);__DC.DebugMessage
#define TRACE_WARN __DC.SetColor (128,0,0);__DC.DebugMessage
#define TRACE_SUCC __DC.SetColor (0,200,0);__DC.DebugMessage

#define TRACEA __DC.SetColor (0, 0, 0);__DC.DebugMessageArea
#define TRACEA_MSG __DC.SetColor (0 ,0 ,0);__DC.DebugMessageArea
#define TRACEA_ERR __DC.SetColor (255,0,0);__DC.DebugMessageArea
#define TRACEA_WARN __DC.SetColor (128,0,0);__DC.DebugMessageArea
#define TRACEA_SUCC __DC.SetColor (0,200,0);__DC.DebugMessageArea

// Set the program name if not the developer has set one
#ifndef DC_PROGRAMNAME
	#define DC_PROGRAMNAME "Unknown"
#endif

 
 
 

// Define Class
class DConsole
{
public:
	// Constructor
	DConsole (char *ProgramName)
	{
		_DebugConsole = NULL;
		_ProgramName = new char [512];
		
		// Copy the Program Name
		strcpy(_ProgramName, ProgramName);
		SetColor(0,128,0);
		DebugMessage ("-- Program start --");
	}

	// Destructor	
	~DConsole()
	{
		SetColor(0,0,128);
		DebugMessage("-- Normal program Exit --");

		if (!_DebugConsole)
			return;
		
		if (_DebugConsole->IsValid ())
		{
			delete _DebugConsole;
			_DebugConsole = NULL;
		}
	}

	// Inits the Messager
	bool InitConsole ()
	{
		// Create the messager
		_DebugConsole = new BMessenger ("application/x-vnd.DebugConsole");
		
		if (!_DebugConsole)
			return false;
		
		if (!_DebugConsole->IsValid ())
			return false;
	
		return true;
	}

	// Safty Checker
	bool CheckDebugConsole ()
	{	
		// If the messager doesn't exist. try to create it
		if (!_DebugConsole)
			return InitConsole();
		
		// Check if it vaild
		if (!_DebugConsole->IsValid())
			return false;

		// All check passed
		return true;
	}
	
	// Message sender	
	void SetColor (int Red, int Green, int Blue)
	{
		ColorRed = Red;
		ColorGreen = Green;
		ColorBlue = Blue;
	}
		
	// Message sender	
	void DebugMessage (char *format, ...)
	{
		// Safty check
		if (!CheckDebugConsole ())
			return;

		// Put it to a string
		va_list Args;
		va_start(Args, format);
		vsprintf(SendBuffer, format, Args);
		va_end(Args);
		
		// Create the message
		BMessage msg (DEBUGCONSOLE_ADDMESSAGE);
		msg.AddString ("DCArea", _ProgramName);
		msg.AddString ("DCMessage", SendBuffer);
		msg.AddInt32 ("DCColorRed", ColorRed);
		msg.AddInt32 ("DCColorGreen", ColorGreen);
		msg.AddInt32 ("DCColorBlue", ColorBlue);

		// Send the message
		_DebugConsole->SendMessage (&msg);
	}

	// Message sender with area selection
	void DebugMessageArea (char *Area, char *format, ...)
	{
		char buf[1024];
		
		sprintf(buf, "%s - %s", _ProgramName, Area);

		// Safty check
		if (!CheckDebugConsole ())
			return;

		// Put it to a string
		va_list Args;
		va_start(Args, format);
		vsprintf(SendBuffer, format, Args);
		va_end(Args);
		
		// Create the message
		BMessage msg (DEBUGCONSOLE_ADDMESSAGE);
		msg.AddString ("DCArea", buf);
		msg.AddString ("DCMessage", SendBuffer);
		msg.AddInt32 ("DCColorRed", ColorRed);
		msg.AddInt32 ("DCColorGreen", ColorGreen);
		msg.AddInt32 ("DCColorBlue", ColorBlue);

		// Send the message
		_DebugConsole->SendMessage (&msg);
	}
	

private:
	// Private variables
	BMessenger *_DebugConsole;
	char *_ProgramName;
	char SendBuffer[2048];
	int ColorRed, ColorGreen, ColorBlue;
};

// Make the Console
DConsole __DC (DC_PROGRAMNAME);

#endif